// Copyright (c) 2005-2006 Masakazu Emi All Rights Reserved.
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;

using Emi.Library.SrcAnalyzerCore.Model;
using Emi.Library.SrcAnalyzerCore.Generator;

namespace Emi.Win.SrcToHtml
{
	public partial class SrcToHtmlForm : Form
	{
		public SrcToHtmlForm()
		{
			InitializeComponent();
			SetupComponent();
		}

		private void SetupComponent()
		{
			//\[X
			srcType.BeginUpdate();
			foreach (string s in SrcXmlToHtml.SourceTypes)
				srcType.Items.Add(s);
			srcType.SelectedIndex = 0;
			srcType.EndUpdate();

			//^uTCY
			Array.ForEach(new int[]{ 2, 4, 8 }, delegate(int size) { tabSize.Items.Add(size); });
			tabSize.SelectedIndex = 1;
		}

		/// <summary>
		/// \[XR[hHTMLւ̕ϊs܂B
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void doConvertToHtml_Click(object sender, EventArgs e)
		{
			string src = srcBox.Text;
			string type = srcType.SelectedItem.ToString();
			int tab = int.Parse(tabSize.SelectedItem.ToString());

			browser.DocumentText = GetHtmlSource(ConvertToCodeBlock(src, type, tab));
		}

		/// <summary>
		/// \[XR[hXMLւ̕ϊs܂B
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void doConvertToXml_Click(object sender, EventArgs e)
		{
			string src = srcBox.Text;
			string type = srcType.SelectedItem.ToString();

			SrcToXmlGenerator g = GeneratorFactory.CreateGenerator(type, src, ".\\xml");
			g.Parse();

			//ςɕ\邯ǂ܂TvƂāB
			browser.DocumentText = g.WriteXml();
		}

		/// <summary>
		/// \pHTML擾܂B
		/// </summary>
		/// <param name="codeBlock"></param>
		/// <returns></returns>
		private static string GetHtmlSource(string codeBlock)
		{
			string head = "";
			string tail = "";
			string css = string.Format("\n<style type=\"text/css\">\n<!--\n{0}-->\n</style>\n", GetCss());
			head = string.Format("<html><head>{0}</head>\n<body>\n<code>", css);
			tail = "</code>\n</body>\n</html>";

			StringBuilder htmlBuilder = new StringBuilder();
			htmlBuilder.Append(head);
			htmlBuilder.Append(codeBlock);
			htmlBuilder.Append(tail);
			return htmlBuilder.ToString();
		}

		/// <summary>
		/// \[XR[h->HTMLR[hւ̕ϊłB
		/// </summary>
		/// <param name="src"></param>
		/// <param name="type"></param>
		/// <param name="tab"></param>
		/// <returns></returns>
		private static string ConvertToCodeBlock(string src, string type, int tab)
		{
			using (StringWriter writer = new StringWriter())
			{
				SrcXmlToHtml converter = new SrcXmlToHtml(src, type, ".\\xml");
				//^uTCY
				converter.TabSize = tab;
				//󔒂u
				converter.IsConvertWhiteSpace = true;
				//sʒu<BR />ǉ
				converter.IsUseBR = true;

				//ϊ
				converter.WriteHtml(writer);
				return writer.ToString();
			}
		}

		private static string GetCss()
		{
			using (StreamReader reader = new StreamReader(".\\xml\\code.css"))
				return reader.ReadToEnd();
		}
	}
}